//genesis cell.g

/*                 functions which appear in this file

=============================================================================
	FUNCTION NAME		ARGUMENTS
=============================================================================
	makecompartment		(path,l,d,Eleak)
	makeneuron		(path,soma_l,soma_d)
=============================================================================
*/



       //==========================================================================
       //                   declare constants
       //==========================================================================
 // initialize the random # generator
 randseed 

 float PI = 3.14159

 // channel equilibrium potentials	mV
 // resting membrane potential
 float EREST = -70e-3
 // leakage potential
 float Eleak =  EREST

 // default jnput level parameters
 float injwidth = 0.05
 float injdelay = 0.01
 float injcurrent = 0.0e-9



//===========================================================================
//                       create compartment
//===========================================================================
function makecompartment(path, l, d, Erest)
    str path
    float l, d
    float Erest
    float area = l*PI*d
    float xarea = PI*d*d/4
    // Ohm-m^2
    float rm = 2
    // F/m^2
    float cm = 0.01
    // ohm-m
    float ra = 2.5

    create compip {path}
    // V
    // ohm
    // F
    // ohm
    setfield {path} Em {Erest} Rm {rm/area} Cm {cm*area} Ra {ra*l/xarea}
end


//===========================================================================
//                       calculate passive current
//===========================================================================
function _calcIc_(action)
	float temp
	temp = {getfield Im} - ({getfield Vm} - {getfield Em}) / {getfield Rm}
	setfield Ic {temp}
	call . PROCESS -parent
end

//===========================================================================
//                   function to create neuron
//===========================================================================
function makeneuron(path, soma_l, soma_d)
    str path
    float soma_l, soma_d

    // 100% of the soma active
    float active_area = soma_l*PI*soma_d*1.0

    create neutral {path}
    pushe {path}
    //=============================================
    //                   cell body
    //=============================================

	// Make extended object for calculating capacitive current
	create compartment tcomp
	addfield tcomp Ic
	addaction tcomp PROCESS _calcIc_
	addobject compip tcomp

    makecompartment soma {soma_l} {soma_d} {Eleak}
    position soma I I R{-soma_l/2.0}


    //=============================================
    //              injection pulse
    //=============================================
    create pulsegen soma/injectpulse
    create diffamp soma/injectpulse/injcurr
    setfield soma/injectpulse/injcurr saturation 1 gain {injcurrent}
    addmsg soma/injectpulse soma/injectpulse/injcurr PLUS output
    addmsg soma/injectpulse/injcurr soma INJECT output
    pope
end
